% Newton's method
%
% ff is the function to optimize
%     called as "[ value, gradient, hessian ] = ff( xx, tt )"
% constraints is a cell array of constraint functions
% initial_xx is the starting point
% initial_tt is the starting log-barrier scale
% mu is the update parameter for tt
% line_search is the line search routine
%     called as "tt = line_search( ff, xx, direction )"
% newton_epsilon is the Newton stopping threshold
% log_barrier_epsilon is the log-barrier stopping threshold
% mm is the dimension of the constraints
%
% xx is the final iterate
% newton_iterations is the a vector of the number of iterations required by each "inner" Newton optimization
%
function [ xx, newton_iterations ] = algorithm_log_barrier( ff, constraints, initial_xx, initial_tt, mu, line_search, newton_epsilon, log_barrier_epsilon, mm )

	maximum_iterations = 65536;

	if ( mu <= 1.0 ); error( 'mu must be greater than one' ); end
	if ( newton_epsilon <= 0.0 ); error( 'newton_epsilon must be positive' ); end
	if ( log_barrier_epsilon <= 0.0 ); error( 'log_barrier_epsilon must be positive' ); end

	phi = @( xx ) objective_scalar_constraints( constraints, xx );

	newton_iterations = [];

	xx = initial_xx;
	tt = initial_tt;

	iterations = 0;
	while true

		newton_ff = @( xx ) objective_log_barrier( ff, phi, xx, tt );
		[ xx, newton_values ] = algorithm_newton( newton_ff, xx, line_search, newton_epsilon );
		newton_iterations = [ newton_iterations, length( newton_values ) - 1 ];

		%tt = ( TODO: update tt );

		%if ( TODO: termination criterion ); break; end

		iterations = iterations + 1;
		if ( iterations > maximum_iterations ); error( 'Too many iterations' ); end

	end

end
